// In this example we create an MQTT client.
// It's publishing messages with topic 'example/hello' and 'Hello World!' payload.
// The MQTT client also subscribes to some topics which are in its interest to receive.
// Ethernet connection is assured by an affordable ENC28J60 Ethernet module.
#include "mbed.h"
#include <UIPEthernet.h>
#include <UIPClient.h>
#include <MQTTClient.h>
#include <string>

Serial  pc(USBTX, USBRX);

// UIPEthernet is the name of a global instance of UIPEthernetClass.
// Do not change the name! It is used within the UIPEthernet library.
// Adapt the SPI pin names to your mbed platform/board if not present yet.
#if defined(TARGET_LPC1768)
UIPEthernetClass    UIPEthernet(p11, p12, p13, p8);         // mosi, miso, sck, cs
#elif defined(TARGET_NUCLEO_F103RB) || defined(TARGET_NUCLEO_L152RE) || defined(TARGET_NUCLEO_F030R8)  \
   || defined(TARGET_NUCLEO_F401RE) || defined(TARGET_NUCLEO_F302R8) || defined(TARGET_NUCLEO_L053R8)  \
   || defined(TARGET_NUCLEO_F411RE) || defined(TARGET_NUCLEO_F334R8) || defined(TARGET_NUCLEO_F072RB)  \
   || defined(TARGET_NUCLEO_F091RC) || defined(TARGET_NUCLEO_F303RE) || defined(TARGET_NUCLEO_F070RB)  \
   || defined(TARGET_KL25Z ) || defined(TARGET_KL46Z) || defined(TARGET_K64F) || defined(TARGET_KL05Z) \
   || defined(TARGET_K20D50M) || defined(TARGET_K22F) \
   || defined(TARGET_NRF51822) \
   || defined(TARGET_RZ_A1H)
UIPEthernetClass    UIPEthernet(D11, D12, D13, D10);        // mosi, miso, sck, cs
#endif

// MAC number must be unique within the connected network. Modify as appropriate.
const uint8_t       MY_MAC[6] = { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05 };

// MQTT broker is like a post office.
// Its task is to distribute messages published by clients to all subscribers (other clients).
// So the 'example/hello' messages published by this client will be sent to the broker.
// Then the broker will send them to all clients which subscribed to such topic (example/hello).
// Also this client will receive all messages with topics it subscribed to (if published such by other clients).
// 'Mosquitto' is a free implementation of MQTT broker for Linux (e.g. Raspberry Pi, Ubuntu etc.)
IPAddress           serverIP(192, 168, 1, 30);  // IP address of your MQTT broker (change to match)
EthernetClient      ethernetClient;
void                onMqttMessage(char* topic, uint8_t* payload, unsigned int length);
MQTTClient          mqttClient(serverIP, 1883, onMqttMessage, ethernetClient);
char                message_buff[100];  // buffer to store received messages

#define DHCP    1   // comment out this line if you'd like to use static IP address

#if !defined(DHCP)
// In case you'd like to use static IP address:
// IP address must be unique and compatible with your network.
// Change as appropriate.
const IPAddress MY_IP(192, 168, 1, 181);
#endif


/**
 * @brief
 * @note
 * @param
 * @retval
 */
int main(void) {
    const int           MAX_TRIES = 5;
    const unsigned long INTERVAL = 5;
    int                 i = 0;
    bool                connected = false;
    char*               payload = "Hello World!";
    unsigned long       t = 0;
    unsigned long       lastTime = t;

    // initialize the ethernet device

#if defined(DHCP)
    pc.printf("Searching for DHCP server..\r\n");

    if(UIPEthernet.begin(MY_MAC) != 1) {
        pc.printf("No DHCP server found.\r\n");
        pc.printf("Exiting application.\r\n");
        return 0;
    }
    pc.printf("DHCP server found and configuration info received.\r\n");
    IPAddress   localIP = UIPEthernet.localIP();
    pc.printf("Local IP = ");
    for(uint8_t i = 0; i < 3; i++)
        pc.printf("%d.", localIP[i]);
    pc.printf("%d\r\n", localIP[3]);
#else
    UIPEthernet.begin(MY_MAC, MY_IP);
#endif

    pc.printf("Connecting to MQTT broker ..\r\n");
    do
    {
        wait(1.0);
        connected = mqttClient.connect("myMQTTHelloClient");
    } while(!connected && (i++ < MAX_TRIES));

    if(connected) {
        pc.printf("MQTT broker connected.\r\n");
        // We can subscribe to various topics published by other clients.
        pc.printf("Subscribing to topics:\r\n");
        pc.printf("    outdoor/temperature\r\n");
        pc.printf("    boiler/outlet/temperature\r\n");
        mqttClient.subscribe("outdoor/temperature");
        mqttClient.subscribe("boiler/outlet/temperature");
    }
    else {
        pc.printf("Failed to connect to MQTT broker.\r\n");
    }

    while(1) {
        t = time(NULL);
        if(t > (lastTime + INTERVAL)) {
            lastTime = t;
            if(connected) {
                pc.printf("Publishing: example/hello\r\n");
                mqttClient.publish("example/hello", payload);
            }
        }
        mqttClient.loop();  // MQTT client loop processing (receiving messages)
    }
}

/**
 * @brief   Called on new MQTT message arrival
 * @note
 * @param   topic:      The topic of the new message
 *          payload:    The payload of the new message
 *          length:     Payload's length
 * @retval
 */
void onMqttMessage(char* topic, uint8_t* payload, unsigned int length) {
    int i = 0;

    pc.printf("Message arrived:\r\n");
    pc.printf("    Topic: %s\r\n", topic);
    pc.printf("    Length: %d\r\n", length);

    // create character buffer with ending null terminator (string)
    for(i = 0; i < length; i++) {
        message_buff[i] = payload[i];
    }

    message_buff[i] = '\0';
    pc.printf("    Payload: %s\r\n", message_buff);
}
